<?php
/**
*
* @package Chat
* @version 0.2
* @copyright (c) 2008 phpBB3.PL Group
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*
*/

/**
 * Chat Message
 *
 * @package Chat
**/
class Chat_Message
{
	public $ID;
	public $text;
	public $time;
	public $author;

	public function __construct(array $row = null)
	{
		if (!is_array($row))
		{
			return;
		}

		$this->ID = $row['message_id'];

		$this->text = new Fields_Textarea_Normal('message');
		$this->text->loadFromDBRow($row);

		$this->time = new Time($row['message_time']);

		$this->author = new UserInfo();
		$this->author->loadFromDBRow($row, 'user', 'guestname');
	}

	public function toDBRow()
	{
		$row = array(
			'message_time' => $this->time->stamp,
			'user_id'      => $this->author->ID,
		);

		if ($this->author->ID == ANONYMOUS)
		{
			$row['guestname'] = $this->author->name;
		}

		$row = $this->text->appendToDBRow($row);

		return $row;
	}

	public function post()
	{
		if (!$this->isAllowed('post'))
		{
			throw new OAPI_Exception('::NotAllowed', 'User');
		}

		global $db;
		$sql = 'INSERT INTO ' . CHAT_TABLE . ' ' . $db->sql_build_array('INSERT', $this->toDBRow());
		$db->sql_query($sql);
	}

	public function saveChanges()
	{
		if (!$this->isAllowed('edit'))
		{
			throw new OAPI_Exception('::NotAllowed', 'User');
		}

		global $db;
		$sql = 'UPDATE ' . CHAT_TABLE . '
					SET ' . $db->sql_build_array('UPDATE', $this->toDBRow()) . '
					WHERE message_id = ' . $this->ID;
		$db->sql_query($sql);
	}

	public function delete()
	{
		if (!$this->isAllowed('delete'))
		{
			throw new OAPI_Exception('::NotAllowed', 'User');
		}

		global $db;
		$sql = 'DELETE FROM ' . CHAT_TABLE . '
					WHERE message_id = ' . $this->ID;
		$db->sql_query($sql);
	}

	public function isAllowed($permission)
	{
		if ($permission == 'post')
		{
			return Chat::isAllowed('u', 'post');
		}

		global $user;
		return (Chat::isAllowed('m', $permission) || (Chat::isAllowed('u', $permission) && $this->author->ID == $user->data['user_id']));
	}
}