<?php
/**
*
* @package Chat
* @version 0.2
* @copyright (c) 2005 phpBB Group, 2008 phpBB3.PL Group
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*
*/

/**
*/
if (!defined('IN_INSTALL'))
{
	// Someone has tried to access the file direct. This is not a good idea, so exit
	exit;
}

if (!empty($setmodules))
{
	// If phpBB is not installed we do not include this module

	if (!file_exists($phpbb_root_path . 'config.' . $phpEx))
	{
		return;
	}

	include_once($phpbb_root_path . 'config.' . $phpEx);

	if (!defined('PHPBB_INSTALLED'))
	{
		return;
	}

	global $language, $lang;
	include($phpbb_root_path . 'language/' . $language . '/mods/install_chat.' . $phpEx);

	$module[] = array(
		'module_type'		=> 'install',
		'module_title'		=> 'INSTALL_CHAT',
		'module_filename'	=> substr(basename(__FILE__), 0, -strlen($phpEx)-1),
		'module_order'		=> 35,
		'module_subs'		=> '',
		'module_stages'		=> array('INTRO', 'INSTALL'),
		'module_reqs'		=> ''
	);
}

/**
* Chat tab
* @package Chat
*/
class install_chat extends module
{
	var $minimumPHPVersion = '5.1.1';

	function install_chat(&$p_master)
	{
		$this->p_master = &$p_master;
	}

	function main($mode, $sub)
	{
		global $lang, $template, $language, $phpbb_root_path;

		$this->page_title = $lang['CHAT_INSTALLATION'];
		$this->tpl_name = 'install_chat';
		$this->mode = $mode;

		$template->assign_var('SUB', $sub);

		switch ($sub)
		{
			case 'intro':
				$this->intro();
			break;

			case 'install':
				$this->install();
			break;
		}
	}

	function link($sub)
	{
		return $this->p_master->module_url . "?mode=$this->mode&amp;sub=$sub";
	}

	function intro()
	{
		global $template, $lang;

		$template->assign_vars(array(
			'S_INTRO'	=> true,

			'TITLE' => $lang['CHAT_INSTALL_INTRO'],
			'BODY'  => $lang['CHAT_INSTALL_INTRO_BODY'],
		));

		if (!$this->intro_OAPI())
		{
			return;
		}

		$this->intro_recommended();

		$template->assign_vars(array(
			'L_SUBMIT' => $lang['NEXT_STEP'],
			'U_ACTION' => $this->link('install'),
		));
	}

	function intro_OAPI()
	{
		global $template, $lang;

		$template->assign_block_vars('section', array(
			'ID'    => 'OAPI',

			'TITLE' => $lang['CHAT_OBJECTIVEAPI'],
			'BODY'  => sprintf($lang['CHAT_OBJECTIVEAPI_BODY'], $this->minimumPHPVersion),
		));

		$phpVersionOK = (version_compare(PHP_VERSION, $this->minimumPHPVersion, '>=')) ? true : false;

		$template->assign_block_vars('section.checks', array(
			'TITLE'   => $lang['CHAT_PHP_VERSION'],
			'VALUE'   => PHP_VERSION,
			'S_VALUE' => $phpVersionOK,
		));

		if (!$phpVersionOK)
		{
			$template->assign_var('UNSUPPORTED_PHP', sprintf($lang['CHAT_PHP_VERSION_TOO_OLD'], $this->minimumPHPVersion));
		}

		return $phpVersionOK;
	}

	function intro_recommended()
	{
		global $template, $lang;

		$template->assign_block_vars('section', array(
			'ID'    => 'recommended',

			'TITLE' => $lang['CHAT_RECOMMENDED'],
			'BODY'  => $lang['CHAT_RECOMMENDED_BODY'],
		));

		foreach (array('XMLWriter') as $class)
		{
			$info = (array) $lang['CHAT_RECOMMENDED_CLASSES'][$class];
			$template->assign_block_vars('section.checks', array(
				'TITLE'   => $info[0],
				'EXPLAIN' => (isset($info[1])) ? $info[1] : '',

				'S_VALUE' => class_exists($class),
				'VALUE'   => $lang[(class_exists($class) ? '' : 'UN') . 'AVAILABLE'],
			));
		}
	}

	function install()
	{
		global $phpbb_root_path, $phpEx;
		include($phpbb_root_path . 'install/InstallationTools.' . $phpEx);
		include($phpbb_root_path . 'install/ChatInstaller.' . $phpEx);

		$installer = new ChatInstaller();

		// PHP4 would produce Parse error, so we have to stringify it and parse only on PHP5.
		eval('
		try
		{
			$installer->install();
		}
		catch (InstallationTools_SQLError $e)
		{
			$this->p_master->db_error($e->getMessage(), $e->sql, __LINE__, __FILE__);
		}
		');

		global $template, $lang;
		$template->assign_vars(array(
			'S_FINAL'	=> true,

			'TITLE'     => $lang['CHAT_INSTALLED'],
			'BODY'      => $lang['CHAT_INSTALLED_BODY'],
		));
	}
}

?>